//
//  KBLinkedFileView.m
//  ------------------
//
//  (c) Keith Blount 2006
//

#import "KBLinkedFileView.h"


@interface KBLinkedFileView (Private)
- (NSDictionary *)fileNameAttributes;
- (NSDictionary *)notFoundAttributes;
- (NSRect)iconRect;
- (NSRect)fileNameRect;
- (BOOL)filePathIsValid;
@end


@implementation KBLinkedFileView

- (id)initWithFrame:(NSRect)frame
{
    if (self = [super initWithFrame:frame])
	{
		filePath = nil;
        backgroundColor = nil;
		fileIcon = nil;
    }
    return self;
}

- (void)dealloc
{
	[filePath release];
	[backgroundColor release];
	[fileIcon release];
	[super dealloc];
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (BOOL)becomeFirstResponder
{
	return YES;
}

- (void)setFilePath:(NSString *)newPath
{
	[newPath retain];
	[filePath release];
	filePath = newPath;
	[self setNeedsDisplay:YES];
	
	// Prepare the icon
	[fileIcon release];
	if (filePath)
	{
		if (fileIcon = [[[NSWorkspace sharedWorkspace] iconForFile:filePath] retain])
			[fileIcon setSize:NSMakeSize(128,128)];
	}
	else
		fileIcon = nil;
}

- (NSString *)filePath
{
	return filePath;
}

- (void)setBackgroundColor:(NSColor *)color
{
	[color retain];
	[backgroundColor release];
	backgroundColor = color;
	[self setNeedsDisplay:YES];
}

- (NSColor *)backgroundColor
{
	return backgroundColor;
}

- (NSDictionary *)fileNameAttributes
{
	return [NSDictionary dictionaryWithObjectsAndKeys:
		[NSColor blueColor], NSForegroundColorAttributeName,
		[NSNumber numberWithInt:1], NSUnderlineStyleAttributeName,
		[NSFont systemFontOfSize:14.0], NSFontAttributeName,
		nil];
}

- (NSDictionary *)notFoundAttributes
{
	return [NSDictionary dictionaryWithObjectsAndKeys:
		[NSColor darkGrayColor], NSForegroundColorAttributeName,
		[NSFont systemFontOfSize:14.0], NSFontAttributeName,
		nil];
}

- (NSRect)iconRect
{
	NSRect bounds = [self bounds];
	NSRect iconRect;
	iconRect.origin.x = roundf((bounds.size.width/2.0) - 64.0) + 0.5;
	iconRect.origin.y = roundf((bounds.size.height/2.0) - 64.0 - 8.0) + 0.5;
	iconRect.size = NSMakeSize(128.0, 128.0);
	return iconRect;
}

- (NSRect)fileNameRect
{
	NSRect bounds = [self bounds];
	NSRect fileNameRect;
	NSSize s = [[filePath lastPathComponent] sizeWithAttributes:[self fileNameAttributes]];
	fileNameRect.origin.x = roundf((bounds.size.width/2.0) - (s.width/2.0)) + 0.5;
	fileNameRect.origin.y = roundf((bounds.size.height/2.0) + 64.0 - 8.0) + 0.5;
	fileNameRect.size = s;
	return fileNameRect;
}

- (BOOL)filePathIsValid
{
	return [[NSFileManager defaultManager] fileExistsAtPath:[self filePath]];
}

- (void)mouseDown:(NSEvent *)theEvent
{
	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	
	if ( [self filePathIsValid] &&
		 ( NSPointInRect(p,[self fileNameRect]) ||
		   (NSPointInRect(p,[self iconRect]) && ([theEvent clickCount] == 2)) ) )
	{
		if (![[NSWorkspace sharedWorkspace] openFile:[self filePath]])
			NSBeep();
		return;
	}
	[super mouseDown:theEvent];
}

- (BOOL)isFlipped
{
	return YES;
}

- (void)drawRect:(NSRect)rect
{
	if (backgroundColor)
	{
		[backgroundColor set];
		[NSBezierPath fillRect:rect];
	}
	
	if (filePath && fileIcon)
	{
		// Draw icon
		NSPoint p = [self iconRect].origin;
		p.y += 128.0;
		[fileIcon compositeToPoint:p operation:NSCompositeSourceOver];
		
		// Draw file name
		NSString *fileName = [filePath lastPathComponent];
		NSDictionary *attribs;
		
		if ([self filePathIsValid])
		{
			attribs = [self fileNameAttributes];
			p = [self fileNameRect].origin;
			[fileName drawAtPoint:p withAttributes:attribs];
		}
		else
		{
			attribs = [self notFoundAttributes];
			NSString *notFoundString = [NSString stringWithFormat:
				NSLocalizedString(@"File '%@' not found",nil), fileName];
			NSRect bounds = [self bounds];
			NSSize s = [notFoundString sizeWithAttributes:attribs];
			p.x = roundf((bounds.size.width/2.0) - (s.width/2.0)) + 0.5;
			[notFoundString drawAtPoint:p withAttributes:attribs];
		}
	}
}

- (void)resetCursorRects
{
	[super resetCursorRects];

	if ([self filePathIsValid])
		[self addCursorRect:[self fileNameRect] cursor:[NSCursor pointingHandCursor]];
}

@end
